import Spot from "./Spot";
import Actor from "../actor/Actor";
import {OptionBuilder} from "../utils/options/OptionBuilder";
import {ISpotlightOptions} from "../utils/options/ISpotlightOptions";
import {Sign} from "../sign/Sign";

export default class Spotlight {
    private readonly spot: Spot;
    private readonly sign: Sign;
    private readonly targets: Actor[];
    private currentActorIndex: number;
    private readonly options: ISpotlightOptions;
    private isSwitching: boolean;

    constructor (options?: ISpotlightOptions) {
        this.targets = [];
        this.currentActorIndex = -1;
        this.options = OptionBuilder.buildOptions(options);
        this.isSwitching = false;
        this.spot = new Spot(this.options);
        this.sign = new Sign(this.options);
    }

    public addActors (...actors: Actor[]): void {
        for (const a of actors) {
            this.targets.push(a);
        }
    }

    public toggleSpot (): boolean {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }

        this.isSwitching = true;
        const on = this.spot.isVisible;
        if (!on) {
            if (this.currentActorIndex === -1) {
                this.currentActorIndex = 0;
            }

            try {
                this._focusActor();
                this.spot.turnOn();
            } catch (error) {
                console.error(error);
            }

        } else {
            if (this.options.resetOnLightsOff) {
                this.currentActorIndex = -1;
            }
            this.spot.turnOff();
            this.sign.hide();
            setTimeout(() => this.isSwitching = false, this.options.delay);
        }
        return on;
    }

    public next (): void {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }

        if (this.spot.isVisible || !this.spot.isVisible && this.options.allowHiddenFocus) {
            this.isSwitching = true;
            if (this.currentActorIndex < this.targets.length-1 && this.currentActorIndex !== -1) {
                this.currentActorIndex++
            }

            else if (this.currentActorIndex === this.targets.length-1 && this.options.loops) {
                this.currentActorIndex = 0;
            }

            else if (this.currentActorIndex === -1) {
                this.currentActorIndex =
                    this.targets.length > 1 ? 1 : 0;
            }

            this._focusActor();
        }
    }

    public previous (): void {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }

        if (this.spot.isVisible || !this.spot.isVisible && this.options.allowHiddenFocus) {
            this.isSwitching = true;
            if (this.currentActorIndex > 0 && this.currentActorIndex !== -1) {
                this.currentActorIndex--;
            } else if (this.currentActorIndex === 0 && this.options.loops) {
                this.currentActorIndex = this.targets.length-1;
            } else if(this.currentActorIndex === -1) {
                this.currentActorIndex = this.targets.length-1;
            }

            this._focusActor();
        }
    }

    private _focusActor (): void {
        if (this.targets.length === 0 || this.currentActorIndex === -1) {
            throw new Error ('No actors have been registered, skipping.');
        }
        if (!this.options.allowHiddenFocus)
            this.spot.turnOn();

        this.sign.hide();
        const actor = this.targets[this.currentActorIndex];

        // launching sign content loading before showing it
        this.sign.setContent(actor);
        this.spot.focus(actor)
            .then(() => {
                this.isSwitching = false;
                this.sign.setHTMLMessage(actor);
            });
    }
}
