import {ISpotlightOptions} from "../utils/options/ISpotlightOptions";
import Actor from "../actor/Actor";
import {SignPosition} from "./SignPosition";
import {IElementInformation} from "../utils/IElementInformation";
import {SignBackground} from "./SignBackground";

export class Sign {
    private readonly root: HTMLElement;
    private readonly options: ISpotlightOptions;
    private readonly background: SignBackground;

    constructor (options: ISpotlightOptions) {
        this.options = options;
        this.background = new SignBackground(options.preventContentClicking);
        this.root = document.createElement('DIV');
        this.root.classList.add('sign', 'hide-sign');
        document.body.appendChild(this.root);
    }

    private isVisible (): boolean {
        return !this.root.classList.contains('hide-sign');
    }

    public hide (): void {
        this.root.classList.add('hide-sign');
        this.background.hide();
    }

    public show (): void {
        this.root.classList.remove('hide-sign');
        this.background.show();
    }

    private _getOptions (actor: Actor): ISpotlightOptions {
        const options = actor.options;
        let nOptions = Object.assign({}, this.options);
        Object.assign(nOptions, options);
        return nOptions;
    }

    setContent (actor: Actor) {
        const nOptions = this._getOptions(actor);

        if (nOptions.signHTML.length !== 0) {
            this.root.innerHTML = nOptions.signHTML;
        } else {
            this.root.innerHTML = `<h4>${nOptions.signTitle}</h4><p>${nOptions.signText}</p>`;
        }
    }

    setHTMLMessage (actor: Actor) {
        const information = actor.getInformation();
        const nOptions = this._getOptions(actor);

        if (!this.isVisible() && nOptions.showSign) {
            this._setPosition(nOptions.signPosition, information);
            this.show();
        }
    }

    private _setPosition (position: SignPosition, information: IElementInformation) {
        const style = getComputedStyle(this.root);
        let height: number = +style.height.substr(0, style.height.length-2);
        let width: number = +style.width.substr(0, style.width.length-2);
        this.root.classList.remove('sign-right', 'sign-left', 'sign-top', 'sign-bottom');

        switch (position) {
            case SignPosition.RIGHT:
                this.root.style.left = information.position.x + information.position.width + 20 + 'px';
                this.root.style.top = information.position.y + information.position.height/2 - +height/2 + 'px';
                this.root.classList.add('sign-right');
                break;
            case SignPosition.LEFT:
                this.root.style.left = +(information.position.x - width) - 2*20 + 'px'; // TODO fix
                this.root.style.top = information.position.y + information.position.height/2 - +height/2 + 'px';
                this.root.classList.add('sign-left');
                break;
            case SignPosition.TOP:
                this.root.style.left = +(information.position.x + information.position.width/2 - width/2) + 'px';
                this.root.style.top = information.position.y - height - 20 + 'px';
                this.root.classList.add('sign-top');
                break;
            case SignPosition.BOTTOM:
                this.root.style.left = +(information.position.x + information.position.width/2 - width/2) + 'px';
                this.root.style.top = information.position.y + information.position.height + 20 + 'px';
                this.root.classList.add('sign-bottom');
                break;
            default:
                throw new Error('not implemented');
        }
    }
}
